/**
 * Easy Coverage for Java
 * <p>
 * Copyright 2011-2012 Tocea Technology Solutions http://www.tocea.com
 * <p>
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
 * <p>
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 * <p>
 * You should have received a copy of the GNU Lesser General Public License along with this library. If not, see
 * <http://www.gnu.org/licenses/>.
 * <p>
 **/

package com.tocea.easycoverage.framework.utils;



import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.List;



/**
 * This type describes a utility class for reflection problems..
 * <p>
 * 
 * @author $Author: sleroy $
 * @since 13 mai 2012
 * @copyright Copyright (C) 2010-2012 TOCEA
 */

public class Utils
{
    
    
    /**
     * This method returns the accessible constructors
     * 
     * @param _class
     *            the class
     * @return the accessible constructors.
     */
    public static List<Constructor<?>> getAccessibleConstructors(final Class<?> _class, final int _modifierToExcludes) {
    
    
        if (_class == null || Modifier.isAbstract(_class.getModifiers())) {
            return new ArrayList<Constructor<?>>();
        }
        final List<Constructor<?>> constructors = new ArrayList<Constructor<?>>();
        for (final Constructor<?> constructor : getConstructors(_class, _modifierToExcludes)) {
            // Skipping constructors with arguments.
            if (constructor.getParameterTypes().length != 0 || constructor.isSynthetic()) {
                continue;
            }
            constructors.add(constructor);
        }
        return constructors;
    }
    
    
    /**
     * This method returns the accessible methods
     * 
     * @param _testedObject
     *            the tested object
     * @param _excludeMethodsModifiers
     *            the modifiers
     * @return the object
     */
    public static List<Method> getAccessibleGetters(final Object _testedObject, final int _excludeMethodsModifiers) {
    
    
        final List<Method> accessibleMethods = getAccessibleMethods(_testedObject, _excludeMethodsModifiers);
        final List<Method> res = new ArrayList<Method>();
        for (final Method method : accessibleMethods) {
            if (method.getParameterTypes().length == 0
                    && (method.getName().startsWith("get") || method.getName().startsWith("is") || method.getName().startsWith("has"))) {
                res.add(method);
            }
            
        }
        return res;
    }
    
    
    /**
     * This method returns the accessible methods of an object (easy to access)
     * 
     * @param _testedObject
     *            the accessible methods
     * @return the accessible methods
     */
    public static List<Method> getAccessibleMethods(final Object _testedObject, final int _modifierToExcludes) {
    
    
        if (_testedObject == null) {
            return new ArrayList<Method>();
        }
        final List<Method> methods = new ArrayList<Method>();
        for (final Method method : getMethods(_testedObject, _modifierToExcludes)) {
            // Skipping constructors with arguments.
            if (method.getParameterTypes().length != 0 || method.isBridge() || method.isSynthetic()) {
                continue;
            }
            
            methods.add(method);
        }
        return methods;
    }
    
    
    /**
     * This method returns the accessible constructors
     * 
     * @param _class
     *            the class
     * @return the accessible constructors.
     */
    public static List<Constructor<?>> getConstructors(final Class<?> _class, final int _modifierToExcludes) {
    
    
        if (_class == null) {
            return new ArrayList<Constructor<?>>();
        }
        final List<Constructor<?>> constructors = new ArrayList<Constructor<?>>();
        for (final Constructor<?> constructor : _class.getDeclaredConstructors()) {
            // Skipping private modifiers
            if ((_modifierToExcludes & constructor.getModifiers()) != 0 || constructor.isSynthetic()) {
                continue;
            }
            constructors.add(constructor);
        }
        return constructors;
    }
    
    
    /**
     * This method collects all the methods of a class.
     * 
     * @param _testedObject
     *            the tested object
     * @param _excludeMethodsModifiers
     *            the modifiers to exclude
     * @return the list of methods
     */
    public static List<Method> getMethods(final Object _testedObject, final int _excludeMethodsModifiers) {
    
    
        if (_testedObject == null) {
            throw new IllegalArgumentException();
        }
        final List<Method> methods = new ArrayList<Method>();
        for (final Method method : _testedObject.getClass().getDeclaredMethods()) {
            // Skipping constructors with arguments.
            // Skipping private modifiers
            if ((_excludeMethodsModifiers & method.getModifiers()) != 0 || method.isBridge() || method.isSynthetic()) {
                continue;
            }
            methods.add(method);
        }
        return methods;
    }
    
    
    /**
     * This method tests if this type is a primitive type.
     * 
     * @param _primitive
     *            the primitive class
     * @return true if the type is a primitive type.
     */
    public static boolean isPrimitiveType(final Class<?> _primitive) {
    
    
        if (_primitive == null) {
            return false;
        }
        if (_primitive.isPrimitive()) {
            return true;
        }
        if (Number.class.isAssignableFrom(_primitive)) {
            return true;
        }
        
        return false;
    }
    
    
    /**
     * Builds a utils.
     */
    private Utils() {
    
    
        super();
    }
}
